==> ppmbutton-2.1.0.tar.gz: ppmbutton-2.1.0/x11.c <==
#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <X11/Xlib.h>

#include "pixmap.h"

extern char *cmnd;

pixmapT *x11RenderString(const char *fontname, const char *string) {
    Display *disp;
    Window root;
    Font font;
    XFontStruct *xfontstruct;
    GC gc;
    int width, height;
    int x, y;
    XImage *img;
    Pixmap pm;
    int border, depth;
    int textheight, textwidth;
    pixmapT *pm2;


    disp = XOpenDisplay(NULL);
    assert(disp);
    XSynchronize(disp, 0);

    root = DefaultRootWindow(disp);
 
    gc = XCreateGC(disp, root, 0, NULL);

    /* get font */
    xfontstruct = XLoadQueryFont(disp, fontname);
    if (!xfontstruct) {
        fprintf(stderr, "%s: cannot load font `%s': %s\n",
                cmnd, fontname, strerror(errno));
        exit(1);
    }
    font = xfontstruct->fid;
    XSetFont(disp, gc, font);

    textheight = xfontstruct->ascent + xfontstruct->descent;
    textwidth = XTextWidth(xfontstruct, string, strlen(string));
    height = (textheight + 4 + 7) / 8 * 8;
    width = (textwidth + 4 + 7) / 8 * 8;

    pm = XCreatePixmap(disp, root, width, height,
                       DefaultDepth(disp, DefaultScreen(disp)));

    /* frame */
    XSetForeground(disp, gc, 0);
    /* left */
    XFillRectangle(disp, pm, gc, 0, 0, width, height);
            
    XSetForeground(disp, gc, 1);
    XDrawString(disp, pm, gc,
                2, 2 + xfontstruct->ascent,
                string, strlen(string));

    XGetGeometry(disp, pm, &root, &x, &y, &width, &height, &border, &depth);

    img = XGetImage(disp, pm, 0, 0, width, height, -1, ZPixmap);
    pm2 = alloc_pixmap(height, width);
    for (y = 0; y < height; y++) {
        for (x = 0; x < width; x++) {
            if (XGetPixel(img, x, y)) {
                pm2->pixels[y][x] = pixel(255, 0, 0, 0);
            }
        }
    }

    return pm2;
}

==> ppmbutton-2.1.5.tar.gz: ppmbutton-2.1.5/x11.c <==
#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <X11/Xlib.h>

#include "pixmap.h"
#include "x11.h"

extern char *cmnd;

pixmapT *x11RenderString(const char *fontname, const char *string) {
    Display *disp;
    Window root;
    Font font;
    XFontStruct *xfontstruct;
    GC gc;
    unsigned int width, height;
    int x, y;
    XImage *img;
    Pixmap pm;
    unsigned int border, depth;
    unsigned int textheight, textwidth;
    pixmapT *pm2;


    disp = XOpenDisplay(NULL);
    assert(disp);
    XSynchronize(disp, 0);

    root = DefaultRootWindow(disp);
 
    gc = XCreateGC(disp, root, 0, NULL);

    /* get font */
    xfontstruct = XLoadQueryFont(disp, fontname);
    if (!xfontstruct) {
        fprintf(stderr, "%s: cannot load font `%s': %s\n",
                cmnd, fontname, strerror(errno));
        exit(1);
    }
    font = xfontstruct->fid;
    XSetFont(disp, gc, font);

    textheight = xfontstruct->ascent + xfontstruct->descent;
    textwidth = XTextWidth(xfontstruct, string, strlen(string));
    height = (textheight + 4 + 7) / 8 * 8;
    width = (textwidth + 4 + 7) / 8 * 8;

    pm = XCreatePixmap(disp, root, width, height,
                       DefaultDepth(disp, DefaultScreen(disp)));

    /* frame */
    XSetForeground(disp, gc, 0);
    /* left */
    XFillRectangle(disp, pm, gc, 0, 0, width, height);
            
    XSetForeground(disp, gc, 1);
    XDrawString(disp, pm, gc,
                2, 2 + xfontstruct->ascent,
                string, strlen(string));

    XGetGeometry(disp, pm, &root, &x, &y, &width, &height, &border, &depth);

    img = XGetImage(disp, pm, 0, 0, width, height, -1, ZPixmap);
    pm2 = alloc_pixmap(height, width);
    for (y = 0; y < height; y++) {
        for (x = 0; x < width; x++) {
            if (XGetPixel(img, x, y)) {
                pm2->pixels[y][x] = pixel(255, 0, 0, 0);
            }
        }
    }

    return pm2;
}
